From: cpw@sgi.com
To: linux-kernel@vger.kernel.org
Cc: mingo@elte.hu, x86@kernel.org
Subject: [patch 4/8 v5] x86, UV: correct reset_with_ipi()
Date: Tue, 21 Jun 2011 07:21:29 -0500 [thread overview]
Message-ID: <20110621122242.497700003@sgi.com> (raw)
In-Reply-To: 20110621122125.688884227@sgi.com
[-- Attachment #1: tlb_uv_ipiresetpnode --]
[-- Type: text/plain, Size: 3082 bytes --]
Fix reset_with_ipi() to look up a cpu for a blade based on the
distribution map being indexed by the potentially sparsely numbered pnode.
This patch is critical to tlb shootdown on a partitioned UV system, or one
with nonconsecutive blade numbers.
Signed-off-by: Cliff Wickman <cpw@sgi.com>
---
The distribution map bits represent pnodes relative to the partition base
pnode. Previous to this patch it had been assuming bits based on 0-based,
consecutive blade ids.
arch/x86/platform/uv/tlb_uv.c | 33 +++++++++++++++++++++------------
1 file changed, 21 insertions(+), 12 deletions(-)
Index: linux/arch/x86/platform/uv/tlb_uv.c
===================================================================
--- linux.orig/arch/x86/platform/uv/tlb_uv.c
+++ linux/arch/x86/platform/uv/tlb_uv.c
@@ -296,14 +296,18 @@ static void bau_process_message(struct m
}
/*
* Determine the first cpu on a uvhub.
*/
-static int uvhub_to_first_cpu(int uvhub)
+static int pnode_to_first_cpu(int pnode, struct bau_control *smaster)
{
int cpu;
- for_each_present_cpu(cpu)
- if (uvhub == uv_cpu_to_blade_id(cpu))
+ struct hub_and_pnode *hpp;
+
+ for_each_present_cpu(cpu) {
+ hpp = &smaster->thp[cpu];
+ if (pnode == hpp->pnode)
return cpu;
+ }
return -1;
}
@@ -366,23 +370,28 @@ static void do_reset(void *ptr)
* Use IPI to get all target uvhubs to release resources held by
* a given sending cpu number.
*/
-static void reset_with_ipi(struct bau_targ_hubmask *distribution, int sender)
+static void reset_with_ipi(struct bau_targ_hubmask *distribution,
+ struct bau_control *bcp)
{
- int uvhub;
+ int pnode;
+ int apnode;
int maskbits;
cpumask_t mask;
+ int sender = bcp->cpu;
+ struct bau_control *smaster = bcp->socket_master;
struct reset_args reset_args;
reset_args.sender = sender;
cpus_clear(mask);
/* find a single cpu for each uvhub in this distribution mask */
maskbits = sizeof(struct bau_targ_hubmask) * BITSPERBYTE;
- for (uvhub = 0; uvhub < maskbits; uvhub++) {
+ /* each bit is a pnode relative to the partition base pnode */
+ for (pnode = 0; pnode < maskbits; pnode++) {
int cpu;
- if (!bau_uvhub_isset(uvhub, distribution))
+ if (!bau_uvhub_isset(pnode, distribution))
continue;
- /* find a cpu for this uvhub */
- cpu = uvhub_to_first_cpu(uvhub);
+ apnode = pnode + bcp->partition_base_pnode;
+ cpu = pnode_to_first_cpu(apnode, smaster);
cpu_set(cpu, mask);
}
@@ -604,7 +613,7 @@ static void destination_plugged(struct b
quiesce_local_uvhub(hmaster);
spin_lock(&hmaster->queue_lock);
- reset_with_ipi(&bau_desc->distribution, bcp->cpu);
+ reset_with_ipi(&bau_desc->distribution, bcp);
spin_unlock(&hmaster->queue_lock);
end_uvhub_quiesce(hmaster);
@@ -626,7 +635,7 @@ static void destination_timeout(struct b
quiesce_local_uvhub(hmaster);
spin_lock(&hmaster->queue_lock);
- reset_with_ipi(&bau_desc->distribution, bcp->cpu);
+ reset_with_ipi(&bau_desc->distribution, bcp);
spin_unlock(&hmaster->queue_lock);
end_uvhub_quiesce(hmaster);
next prev parent reply other threads:[~2011-06-21 12:24 UTC|newest]
Thread overview: 17+ messages / expand[flat|nested] mbox.gz Atom feed top
2011-06-21 12:21 [patch 0/8 v5] Subject: x86, UV: fixes to tlb shootdown cpw
2011-06-21 12:21 ` [patch 1/8 v5] x86, UV: smp_processor_id in a preemptable region cpw
2011-06-21 15:16 ` [tip:x86/uv] x86, UV: Fix smp_processor_id() use " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 2/8 v5] x86, UV: inline header file functions cpw
2011-06-21 15:16 ` [tip:x86/uv] x86, UV: Inline " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 3/8 v5] x86, UV: allow for non-consecutive sockets cpw
2011-06-21 15:17 ` [tip:x86/uv] x86, UV: Allow " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` cpw [this message]
2011-06-21 15:17 ` [tip:x86/uv] x86, UV: Correct reset_with_ipi() tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 5/8 v5] x86, UV: rename hubmask to pnmask cpw
2011-06-21 15:18 ` [tip:x86/uv] x86, UV: Rename " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 6/8 v5] x86, UV: remove cpumask_t from the stack cpw
2011-06-21 15:18 ` [tip:x86/uv] x86, UV: Remove " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 7/8 v5] x86, UV: correct failed topology memory leak cpw
2011-06-21 15:18 ` [tip:x86/uv] x86, UV: Correct " tip-bot for cpw@sgi.com
2011-06-21 12:21 ` [patch 8/8 v5] x86, UV: correct UV2 BAU destination timeout cpw
2011-06-21 15:19 ` [tip:x86/uv] x86, UV: Correct " tip-bot for cpw@sgi.com
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20110621122242.497700003@sgi.com \
--to=cpw@sgi.com \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
--cc=x86@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -296,14 +296,18 @@ static void bau_process_message(struct m
}
/*
- * Determine the first cpu on a uvhub.
+ * Determine the first cpu on a pnode.
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.